import argparse
import sys

from chromie.commands import init, package, preview


def parse_args(args):
    parser = argparse.ArgumentParser(prog="chromie")

    subparsers = parser.add_subparsers(
        help="desired command to perform", dest="command"
    )

    parent_parser = argparse.ArgumentParser(add_help=False)
    parent_parser.add_argument(
        "filepath", help="project directory", action="store", type=str
    )

    init_parser = subparsers.add_parser(
        "init", parents=[parent_parser], help="initialize project directory"
    )

    init_parser.add_argument(
        "-n", "--name", help="specify the project name", action="store", type=str
    )

    init_parser.add_argument(
        "-o",
        "--overwrite",
        help="overwrite files if directory already exists",
        action="store_true",
    )

    package_parser = subparsers.add_parser(
        "package", parents=[parent_parser], help="package project directory"
    )

    package_parser.add_argument(
        "-i",
        "--increment-version",
        help="increment project version before packaging based with (major, minor, patch) as options",
        action="store",
    )

    preview_parser = subparsers.add_parser(
        "preview", parents=[parent_parser], help="preview project in browser"
    )
    return parser.parse_args(args)


def main():

    args = parse_args(sys.argv[1:])

    if args.command == "init":
        init(args)

    elif args.command == "package":
        package(args)

    elif args.command == "preview":
        preview(args)

    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import os
import sys
import json
import shutil

from chromie.utils import ChromiePathFinder, ManifestFile


NAME_PROMPT = "What is the name of your project?\nname: "
OVERWRIGHT_PROMPT = (
    "This directory with this name already exists.\n"
    "Would you like to overwrite anyway? Y/N: "
)
AFFERMATIVE = (
    "Y",
    "YES",
)
NEGATIVE = (
    "N",
    "NO",
)


def init(args):

    name = args.name if args.name else input(NAME_PROMPT)
    exist_ok = args.overwrite

    finder = ChromiePathFinder(args.filepath, name)

    if not exist_ok and os.path.exists(finder.root):
        asked = 0
        overwrite_prompt = ""
        while asked <= 3 or overwrite_prompt not in [*AFFERMATIVE, *NEGATIVE]:
            overwrite_prompt = input(OVERWRIGHT_PROMPT).upper()
            asked += 1
            if overwrite_prompt in NEGATIVE or asked >= 3:
                raise SystemExit()

            elif overwrite_prompt in AFFERMATIVE:
                shutil.rmtree(finder.root, ignore_errors=True)
                break
    else:
        shutil.rmtree(finder.root, ignore_errors=True)

    if not os.path.exists(finder.root):

        os.makedirs(finder("web_store"))
        os.makedirs(finder("images"))

        with open(finder("gitignore"), "w") as f:
            f.write("")

        with open(finder("zipignore"), "w") as f:
            f.writelines(f"\n".join((".zipignore", "dist")))

        ManifestFile(
            finder("manifest"),
            {"name": name, "manifest_version": 2, "version": "0.1.0"},
        ).write()



from enum import Enum
import os
import ntpath
import json


class Paths(Enum):
    gitignore = ".gitignore"
    zipignore = ".zipignore"
    dist = "dist/"
    web_store = "dist/web store/"
    src = "src/"
    images = "src/images"
    manifest = "src/manifest.json"


class ChromiePathFinder(dict):
    def __init__(self, path=os.getcwd(), name=None, paths=Paths):
        if not name:
            path, name = os.path.split(path)
        self.path = path
        self.name = name
        self.paths = paths

    @property
    def root(self):
        return os.path.join(self.path, self.name)

    def __call__(self, name):
        dir = Paths[name].value
        if not dir:
            raise ValueError(f"No path with the name {name} exists")
        return os.path.abspath(os.path.join(self.root, dir))


class ManifestFile:
    def __init__(self, path, data):
        self.path = path
        self.data = data

    @classmethod
    def from_file(cls, path):
        with open(path, "r") as f:
            data = json.load(f)
        return cls(path, data)

    def write(self, data=None):
        data = self.data if not data else data
        with open(self.path, "w") as f:
            json.dump(data, f, indent=2)

    def increment_version(self, vers):

        current_version = self.data["version"]

        major, minor, patch = current_version.split(".")

        if vers == "major":
            major = str(int(major) + 1)

        elif vers == "minor":
            minor = str(int(minor) + 1)

        elif vers == "patch":
            patch = str(int(patch) + 1)

        new_version = ".".join((major, minor, patch))

        self.data["version"] = current_version.replace(current_version, new_version)

        self.write()



import os
import sys
import json
from fnmatch import fnmatch
import zipfile

from chromie.utils import ChromiePathFinder, ManifestFile

VERSION_PROMPT = "How would you like to increment the version?\nOptions are either 'major', 'minor', or 'patch': "
INVALID_VERSION_ARGUMENT = (
    "Incorrect version was entered. Please enter either 'major', 'minor' or 'patch'."
)
NO_DIST_FOUND = "No dist directory was found in the directory specified."
ZIP_SUCCESSFUL = "{} was packaged successfully!"


def is_valid_version_arg(increment_version):
    if increment_version and increment_version.lower() in ("minor", "major", "patch"):
        return True
    return False


def package(args):
    # TODO: impliment --version -v argument for setting the version in manifest.json

    filepath = os.path.abspath(args.filepath)
    increment_version = args.increment_version

    if not increment_version:
        increment_version = input(VERSION_PROMPT)

    if not is_valid_version_arg(increment_version):
        raise SystemExit(INVALID_VERSION_ARGUMENT)

    finder = ChromiePathFinder(filepath)
    manifest = ManifestFile.from_file(finder("manifest"))
    manifest.increment_version(increment_version)

    zipignore_file = finder("zipignore")
    ignore = [".gitignore", "dist", ".DS_Store"]

    dist = finder("dist")
    if not os.path.isdir(dist):
        os.mkdir(dist)

    directory = os.listdir(filepath)

    with open(zipignore_file, "r") as zipignore:
        lines = [line.rstrip() for line in zipignore.readlines()]
        [ignore.append(n) for n in directory for l in lines if fnmatch(n, l)]

    with zipfile.ZipFile(
        os.path.join(dist, f"{finder.name}.zip"), "w", zipfile.ZIP_DEFLATED
    ) as zip:

        def write_zip(fp, root, name):
            absname = os.path.abspath(os.path.join(root, name))
            arcname = absname[len(fp) + 1 :]
            zip.write(absname, arcname)

        for item in directory:
            if item not in ignore:
                for dirname, subdirs, files in os.walk(os.path.join(filepath, item)):
                    [write_zip(filepath, dirname, fn) for fn in files]
                    [write_zip(filepath, dirname, d) for d in subdirs]



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="chromie",
    version="0.1.1",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Sandersland/chromie",
    author="Steffen Andersland",
    author_email="stefandersland@gmail.com",
    license="MIT",
    extras_require={"dev": ["twine"]},
    entry_points={"console_scripts": ["chromie=chromie:main"]},
)
